@@ -28,11 +28,7 @@ class SYSTEMSETTINGSVIEW_EXPORT MenuProxyModel : public KCategorizedSortFilterPr
 {
     Q_OBJECT
 
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-    Q_PROPERTY(QString filterRegExp READ filterRegExp WRITE setFilterRegExp NOTIFY filterRegExpChanged)
-#else
     Q_PROPERTY(QString filterRegExp READ filterRegularExpression WRITE setFilterRegularExpression NOTIFY filterRegularExpressionChanged)
-#endif
 
 public:
     /**
@@ -83,21 +79,6 @@ public:
      */
     Qt::ItemFlags flags(const QModelIndex &index) const override;
 
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-    /**
-     * Please see Qt QAbstractItemModel documentation for more details.\n
-     * Reimplemented for internal reasons.
-     */
-    void setFilterRegExp(const QRegExp &regExp);
-
-    /**
-     * Please see Qt QAbstractItemModel documentation for more details.\n
-     * Reimplemented for internal reasons.
-     */
-    void setFilterRegExp(const QString &pattern);
-
-    QString filterRegExp() const;
-#else
     /**
      * Please see Qt QAbstractItemModel documentation for more details.\n
      * Reimplemented for internal reasons.
@@ -111,7 +92,6 @@ public:
     void setFilterRegularExpression(const QString &pattern);
 
     QString filterRegularExpression() const;
-#endif
 
     /**
      * makes the filter highlight matching entries instead of hiding them
@@ -124,11 +104,7 @@ public:
     bool filterHighlightsEntries() const;
 
 Q_SIGNALS:
-#if QT_VERSION < QT_VERSION_CHECK(6, 0, 0)
-    void filterRegExpChanged();
-#else
     void filterRegularExpressionChanged();
-#endif
 
 private:
     bool m_filterHighlightsEntries : 1;
