@@ -24,24 +24,17 @@
 #include <QWhatsThis>
 
 #include <KAboutData>
-#include <kauth_version.h>
-#if KAUTH_VERSION >= QT_VERSION_CHECK(5, 92, 0)
 #include <KAuth/Action>
-#include <KAuth/ObjectDecorator>
-#else
-#include <KAuthAction>
-#include <KAuthObjectDecorator>
-#endif
+#include <KAuth/ExecuteJob>
 #include <KAuthorized>
-#include <KCModuleInfo>
-#include <KCModuleProxy>
+#include <KCModule>
+#include <KCModuleLoader>
 #include <KColorScheme>
 #include <KMessageDialog>
 #include <KPageWidget>
 #include <KSharedConfig>
 #include <KStandardGuiItem>
 #include <KTitleWidget>
-#include <Kirigami/Units>
 
 #include <KActivities/ResourceInstance>
 
@@ -64,7 +57,10 @@ CustomTitle::CustomTitle(QWidget *parent)
     : KTitleWidget(parent)
 {
     // Use the same left margin as QML titles for consistency (Kirigami/AbstractPageHeader.qml)
-    setContentsMargins(Kirigami::Units().gridUnit(),
+    // 18px is Standard Kirigami gridUnit for 10pt Noto Sans.
+    // TODO: make this use a real gridUnit so it will adjust to the user's font,
+    // once we have a QmlEngine object such that  using it won't risk crashes!
+    setContentsMargins(18,
                        style()->pixelMetric(QStyle::PM_LayoutTopMargin),
                        style()->pixelMetric(QStyle::PM_LayoutRightMargin),
                        style()->pixelMetric(QStyle::PM_LayoutBottomMargin));
@@ -120,29 +116,30 @@ public:
     Private()
     {
     }
-    QMap<KPageWidgetItem *, KCModuleProxy *> mPages;
+    QMap<KPageWidgetItem *, KCModule *> mPages;
     QMap<KPageWidgetItem *, QString> mPagesPluginIdMap;
     KPageWidget *mPageWidget = nullptr;
     CustomTitle *mCustomHeader = nullptr;
     QVBoxLayout *mLayout = nullptr;
     QDialogButtonBox *mButtons = nullptr;
-    KAuth::ObjectDecorator *mApplyAuthorize = nullptr;
     QPushButton *mApply = nullptr;
     QPushButton *mReset = nullptr;
     QPushButton *mDefault = nullptr;
     QPushButton *mHelp = nullptr;
+    QIcon mApplyIcon;
     KMessageDialog *mResolvingChangesDialog = nullptr;
     bool pageChangeSupressed = false;
     bool mSaveStatistics = true;
     bool mDefaultsIndicatorsVisible = false;
     KCModule::Buttons mButtonMask = ~KCModule::Buttons(KCModule::NoAdditionalButton);
+    KAuth::Action authAction;
 };
 
 ModuleView::ModuleView(QWidget *parent)
     : QWidget(parent)
     , d(new Private())
 {
-    QVBoxLayout *rootLayout = new QVBoxLayout(this);
+    auto rootLayout = new QVBoxLayout(this);
     rootLayout->setContentsMargins(0, 0, 0, 0);
     rootLayout->setSpacing(0);
     // Configure a layout first
@@ -158,7 +155,7 @@ ModuleView::ModuleView(QWidget *parent)
     d->mPageWidget->layout()->setContentsMargins(0, 0, 0, 0);
 
     // Zero out only the horizontal spacing (the vertical spacing is fine)
-    QGridLayout *gridLayout = static_cast<QGridLayout *>(d->mPageWidget->layout());
+    auto gridLayout = static_cast<QGridLayout *>(d->mPageWidget->layout());
 
     gridLayout->setHorizontalSpacing(0);
 
@@ -169,6 +166,7 @@ ModuleView::ModuleView(QWidget *parent)
 
     // Create the buttons in it
     d->mApply = d->mButtons->addButton(QDialogButtonBox::Apply);
+    d->mApplyIcon = d->mApply->icon();
     KGuiItem::assign(d->mApply, KStandardGuiItem::apply());
     d->mDefault = d->mButtons->addButton(QDialogButtonBox::RestoreDefaults);
     KGuiItem::assign(d->mDefault, KStandardGuiItem::defaults());
@@ -197,8 +195,15 @@ ModuleView::ModuleView(QWidget *parent)
     connect(d->mPageWidget, SIGNAL(currentPageChanged(KPageWidgetItem*,KPageWidgetItem*)),
              this, SLOT(activeModuleChanged(KPageWidgetItem*,KPageWidgetItem*)));
     // clang-format on
-    d->mApplyAuthorize = new KAuth::ObjectDecorator(d->mApply);
-    d->mApplyAuthorize->setAuthAction(KAuth::Action());
+
+    connect(d->mApply, &QPushButton::clicked, this, [this] {
+        if (d->authAction.isValid()) {
+            KAuth::ExecuteJob *job = d->authAction.execute(KAuth::Action::AuthorizeOnlyMode);
+            connect(job, &KAuth::ExecuteJob::statusChanged, this, [this](KAuth::Action::AuthStatus status) {
+                authStatusChanged(status);
+            });
+        }
+    });
 }
 
 ModuleView::~ModuleView()
@@ -217,7 +222,7 @@ void ModuleView::loadModule(const QModelIndex &menuItem, const QStringList &args
         return;
     }
 
-    MenuItem *item = menuItem.data(Qt::UserRole).value<MenuItem *>();
+    auto item = menuItem.data(Qt::UserRole).value<MenuItem *>();
 
     // if module has a main page (like in Appearance > Global Theme) we'll load that
     if (item->isLibrary() || item->isExternalAppModule()) {
@@ -225,7 +230,7 @@ void ModuleView::loadModule(const QModelIndex &menuItem, const QStringList &args
     }
     // if module doesn't have a main page, we'll load the first subpage
     else if (menuItem.model()->rowCount(menuItem) > 0) {
-        MenuItem *subpageItem = menuItem.model()->index(0, 0, menuItem).data(Qt::UserRole).value<MenuItem *>();
+        auto subpageItem = menuItem.model()->index(0, 0, menuItem).data(Qt::UserRole).value<MenuItem *>();
         addModule(subpageItem, args);
     }
 }
@@ -241,35 +246,36 @@ void ModuleView::addModule(MenuItem *item, const QStringList &args)
         return;
     }
 
-    if (KPageWidgetItem *page = d->mPagesPluginIdMap.key(data.name())) {
+    if (KPageWidgetItem *page = d->mPagesPluginIdMap.key(data.pluginId())) {
         activeModuleChanged(page, d->mPageWidget->currentPage());
         return;
     }
 
     // Create the scroller
-    auto *moduleScroll = new QScrollArea(this);
+    auto moduleScroll = new QScrollArea(this);
     // Prepare the scroll area
     moduleScroll->setWidgetResizable(true);
     moduleScroll->setFrameStyle(QFrame::NoFrame);
     moduleScroll->viewport()->setAutoFillBackground(false);
     // Create the page
-    auto *page = new KPageWidgetItem(moduleScroll, data.name());
+    auto page = new KPageWidgetItem(moduleScroll, data.name());
     // Provide information to the users
 
     if (item->isExternalAppModule()) {
-        auto *externalWidget = new ExternalAppModule(this, KService::Ptr(new KService(item->metaData().metaDataFileName())));
+        auto externalWidget = new ExternalAppModule(KService::Ptr(new KService(item->metaData().fileName())));
         moduleScroll->setWidget(externalWidget);
         d->mCustomHeader->setText(item->metaData().name()); // We have to set this manually, BUG: 448672
         page->setName(QString());
     } else { // It must be a normal module then
-        auto *moduleProxy = new KCModuleProxy(data, moduleScroll, args);
-        moduleScroll->setWidget(moduleProxy);
-        moduleProxy->setAutoFillBackground(false);
-        connect(moduleProxy, &KCModuleProxy::changed, this, &ModuleView::stateChanged);
-        d->mPages.insert(page, moduleProxy);
+        auto kcm = KCModuleLoader::loadModule(data, moduleScroll, QVariantList(args.begin(), args.end()));
+        moduleScroll->setWidget(kcm->widget());
+        kcm->widget()->setAutoFillBackground(false);
+        kcm->load();
+        connect(kcm, &KCModule::needsSaveChanged, this, &ModuleView::stateChanged);
+        d->mPages.insert(page, kcm);
     }
 
-    d->mPagesPluginIdMap.insert(page, data.name());
+    d->mPagesPluginIdMap.insert(page, data.pluginId());
     updatePageIconHeader(page);
     // Add the new page
     d->mPageWidget->addPage(page);
@@ -282,17 +288,17 @@ void ModuleView::updatePageIconHeader(KPageWidgetItem *page)
         return;
     }
 
-    KCModuleProxy *moduleProxy = d->mPages.value(page);
-    if (!moduleProxy || !moduleProxy->metaData().isValid()) {
+    KCModule *kcm = d->mPages.value(page);
+    if (!kcm || !kcm->metaData().isValid()) {
         // Seems like we have some form of a race condition going on here...
         return;
     }
 
-    const QString moduleName = moduleProxy->metaData().name();
+    const QString moduleName = kcm->metaData().name();
     page->setHeader(moduleName);
-    page->setIcon(QIcon::fromTheme(moduleProxy->metaData().iconName()));
+    page->setIcon(QIcon::fromTheme(kcm->metaData().iconName()));
 
-    const bool isQml = moduleProxy->realModule() && moduleProxy->realModule()->inherits("KCModuleQml");
+    const bool isQml = kcm->inherits("KCModuleQml");
     const bool isSidebar = faceType() == KPageView::Plain;
 
     // Use the module's header only for QWidgets KCMs on Icons mode
@@ -311,13 +317,13 @@ void ModuleView::updatePageIconHeader(KPageWidgetItem *page)
 
 bool ModuleView::resolveChanges()
 {
-    KCModuleProxy *currentProxy = d->mPages.value(d->mPageWidget->currentPage());
-    return resolveChanges(currentProxy);
+    KCModule *kcm = d->mPages.value(d->mPageWidget->currentPage());
+    return resolveChanges(kcm);
 }
 
-bool ModuleView::resolveChanges(KCModuleProxy *currentProxy)
+bool ModuleView::resolveChanges(KCModule *kcm)
 {
-    if (!currentProxy || !currentProxy->isChanged()) {
+    if (!kcm || !kcm->needsSave()) {
         return true;
     }
 
@@ -340,9 +346,9 @@ bool ModuleView::resolveChanges(KCModuleProxy *currentProxy)
 
     switch (result) {
     case KMessageDialog::PrimaryAction:
-        return moduleSave(currentProxy);
+        return moduleSave(kcm);
     case KMessageDialog::SecondaryAction:
-        currentProxy->load();
+        kcm->load();
         return true;
     case KMessageDialog::Cancel:
         return false;
@@ -355,8 +361,10 @@ bool ModuleView::resolveChanges(KCModuleProxy *currentProxy)
 void ModuleView::closeModules()
 {
     d->pageChangeSupressed = true;
-    d->mApplyAuthorize->setAuthAction(KAuth::Action()); // Ensure KAuth knows that authentication is now pointless...
+    d->authAction = KAuth::Action();
     for (auto page = d->mPagesPluginIdMap.cbegin(); page != d->mPagesPluginIdMap.cend(); ++page) {
+        // Delete the KCM first, because e.g. the KFontInst KCM accesses it's widgets in the destructor
+        delete d->mPages.value(page.key());
         d->mPageWidget->removePage(page.key());
     }
 
@@ -367,11 +375,11 @@ void ModuleView::closeModules()
 
 bool ModuleView::moduleSave()
 {
-    KCModuleProxy *moduleProxy = d->mPages.value(d->mPageWidget->currentPage());
+    KCModule *moduleProxy = d->mPages.value(d->mPageWidget->currentPage());
     return moduleSave(moduleProxy);
 }
 
-bool ModuleView::moduleSave(KCModuleProxy *module)
+bool ModuleView::moduleSave(KCModule *module)
 {
     if (!module) {
         return false;
@@ -384,7 +392,7 @@ bool ModuleView::moduleSave(KCModuleProxy *module)
 
 void ModuleView::moduleLoad()
 {
-    KCModuleProxy *activeModule = d->mPages.value(d->mPageWidget->currentPage());
+    KCModule *activeModule = d->mPages.value(d->mPageWidget->currentPage());
     if (activeModule) {
         activeModule->load();
     }
@@ -392,7 +400,7 @@ void ModuleView::moduleLoad()
 
 void ModuleView::moduleDefaults()
 {
-    KCModuleProxy *activeModule = d->mPages.value(d->mPageWidget->currentPage());
+    KCModule *activeModule = d->mPages.value(d->mPageWidget->currentPage());
     if (activeModule) {
         activeModule->defaults();
     }
@@ -400,7 +408,7 @@ void ModuleView::moduleDefaults()
 
 void ModuleView::moduleHelp()
 {
-    KCModuleProxy *activeModule = d->mPages.value(d->mPageWidget->currentPage());
+    KCModule *activeModule = d->mPages.value(d->mPageWidget->currentPage());
     if (!activeModule) {
         return;
     }
@@ -419,7 +427,7 @@ void ModuleView::activeModuleChanged(KPageWidgetItem *current, KPageWidgetItem *
 {
     d->mPageWidget->blockSignals(true);
     d->mPageWidget->setCurrentPage(previous);
-    KCModuleProxy *previousModule = d->mPages.value(previous);
+    KCModule *previousModule = d->mPages.value(previous);
     if (resolveChanges(previousModule)) {
         d->mPageWidget->setCurrentPage(current);
     }
@@ -431,7 +439,7 @@ void ModuleView::activeModuleChanged(KPageWidgetItem *current, KPageWidgetItem *
     // We need to get the state of the now active module
     stateChanged();
 
-    KCModuleProxy *activeModule = d->mPages.value(d->mPageWidget->currentPage());
+    KCModule *activeModule = d->mPages.value(d->mPageWidget->currentPage());
     if (!activeModule) {
         return;
     }
@@ -450,7 +458,10 @@ void ModuleView::activeModuleChanged(KPageWidgetItem *current, KPageWidgetItem *
                                     style()->pixelMetric(QStyle::PM_LayoutBottomMargin));
     d->mPageWidget->layout()->setSpacing(0);
     if (auto titleWidget = qobject_cast<KTitleWidget *>(d->mPageWidget->pageHeader())) {
-        titleWidget->layout()->setContentsMargins(Kirigami::Units().gridUnit(),
+        // 18px is Standard Kirigami gridUnit for 10pt Noto Sans.
+        // TODO: make this use a real gridUnit so it will adjust to the user's font,
+        // once we have a QmlEngine object such that  using it won't risk crashes!
+        titleWidget->layout()->setContentsMargins(18,
                                                   style()->pixelMetric(QStyle::PM_LayoutRightMargin),
                                                   style()->pixelMetric(QStyle::PM_LayoutRightMargin),
                                                   style()->pixelMetric(QStyle::PM_LayoutBottomMargin));
@@ -465,39 +476,36 @@ void ModuleView::stateChanged()
     updatePageIconHeader(d->mPageWidget->currentPage());
     updateButtons();
 
-    KCModuleProxy *activeModule = d->mPages.value(d->mPageWidget->currentPage());
-    Q_EMIT moduleChanged(activeModule && activeModule->isChanged());
+    KCModule *activeModule = d->mPages.value(d->mPageWidget->currentPage());
+    Q_EMIT moduleChanged(activeModule && activeModule->needsSave());
 }
 
 void ModuleView::updateButtons()
 {
-    KCModuleProxy *activeModule = d->mPages.value(d->mPageWidget->currentPage());
-    KAuth::Action moduleAction;
+    KCModule *activeModule = d->mPages.value(d->mPageWidget->currentPage());
     bool change = false;
     bool defaulted = false;
     KCModule::Buttons buttons = KCModule::NoAdditionalButton;
 
     if (activeModule) {
         buttons = activeModule->buttons() & d->mButtonMask;
-        change = activeModule->isChanged();
-        defaulted = activeModule->defaulted();
+        change = activeModule->needsSave();
+        defaulted = activeModule->representsDefaults();
+
+        d->authAction = KAuth::Action(activeModule->authActionName());
+        authStatusChanged(d->authAction.status());
 
         // Do not display Help button if there is no docPath available
         if (activeModule->metaData().value(QStringLiteral("X-DocPath")).isEmpty()) {
             buttons &= ~KCModule::Help;
         }
 
-        disconnect(d->mApplyAuthorize, SIGNAL(authorized(KAuth::Action)), this, SLOT(moduleSave()));
         disconnect(d->mApply, SIGNAL(clicked()), this, SLOT(moduleSave()));
-        if (activeModule->realModule()->authAction().isValid()) {
-            connect(d->mApplyAuthorize, SIGNAL(authorized(KAuth::Action)), this, SLOT(moduleSave()));
-            moduleAction = activeModule->realModule()->authAction();
-        } else {
-            connect(d->mApply, SIGNAL(clicked()), this, SLOT(moduleSave()));
-        }
+        connect(d->mApply, SIGNAL(clicked()), this, SLOT(moduleSave()));
+    } else {
+        d->authAction = KAuth::Action();
     }
 
-    d->mApplyAuthorize->setAuthAction(moduleAction);
     d->mDefault->setEnabled(!defaulted);
     d->mDefault->setVisible(buttons & KCModule::Default);
     d->mApply->setEnabled(change);
@@ -579,7 +587,7 @@ bool ModuleView::isResetVisible() const
 void ModuleView::moduleShowDefaultsIndicators(bool show)
 {
     d->mDefaultsIndicatorsVisible = show;
-    KCModuleProxy *activeModule = d->mPages.value(d->mPageWidget->currentPage());
+    KCModule *activeModule = d->mPages.value(d->mPageWidget->currentPage());
     if (activeModule) {
         activeModule->setDefaultsIndicatorsVisible(show);
     }
@@ -599,22 +607,49 @@ qreal ModuleView::headerHeight() const
     return d->mCustomHeader->minimumHeight();
 }
 
-void ModuleView::setActiveModule(const QString &moduleName)
+void ModuleView::setActiveModule(const QString &pluginId)
 {
     const auto pageList = d->mPagesPluginIdMap.keys();
     for (const auto page : pageList) {
-        if (d->mPagesPluginIdMap.value(page) == moduleName) {
+        if (d->mPagesPluginIdMap.value(page) == pluginId) {
             d->mPageWidget->setCurrentPage(page);
             break;
         }
     }
 }
 
+void ModuleView::requestActivation(const QVariantList &args)
+{
+    KCModule *activeModule = d->mPages.value(d->mPageWidget->currentPage());
+    if (activeModule) {
+        Q_EMIT activeModule->activationRequested(args);
+    }
+}
+
 KPluginMetaData ModuleView::activeModuleMetadata() const
 {
-    KCModuleProxy *activeModule = d->mPages.value(d->mPageWidget->currentPage());
+    KCModule *activeModule = d->mPages.value(d->mPageWidget->currentPage());
     if (!activeModule) {
         return {};
     }
     return activeModule->metaData();
 }
+
+void ModuleView::authStatusChanged(KAuth::Action::AuthStatus status)
+{
+    switch (status) {
+    case KAuth::Action::AuthorizedStatus:
+        d->mApply->setEnabled(true);
+        d->mApply->setIcon(d->mApplyIcon);
+        break;
+    case KAuth::Action::AuthRequiredStatus:
+        d->mApply->setEnabled(true);
+        d->mApply->setIcon(QIcon::fromTheme(QStringLiteral("dialog-password")));
+        break;
+    default:
+        d->mApply->setEnabled(false);
+        d->mApply->setIcon(d->mApplyIcon);
+    }
+}
+
+#include "moc_ModuleView.cpp"
